# Autograder for CS2303 MQP, 2018-2019 Academic Year, Worcester Polytechnic Institute
# Authors: Chang Liu, Tim Whitworth
# Autograder for Assignment 1
import os, subprocess, random, difflib

# Main program for the autograder
def main():

	# Print the welcome message
    print("Autograder for HW1:\n")

    # Call 'make' in the Eclipse Debug directory
    print("Calling 'make'...\n")
    try:
    	# Call 'make clean' to remove any outdated files
        print(subprocess.run("make clean -C " + os.path.dirname(os.path.abspath(__file__)) + "/../HW1/Debug", stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True, shell=True, universal_newlines=True).stdout)
    except subprocess.CalledProcessError as err:
        print(err.stderr, end='')
        return
    try:
    	# Compile the student's code into an executable called 'HW1'
        print(subprocess.run("make -C " + os.path.dirname(os.path.abspath(__file__)) + "/../HW1/Debug", stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True, shell=True, universal_newlines=True).stdout)
    except subprocess.CalledProcessError as err:
        print(err.stderr, end='')
        return
	# Compiling finished
    print("'make' called successfully!\n")

    # Generate 10 test cases
    four_hundred_years = list(range(2000, 10000, 400))
    one_hundred_years = [year for year in range(1800, 10000, 100) if year not in range(2000, 10000, 400)]
    four_years = [year for year in range(1752, 10000, 4) if year not in range(1800, 10000, 100)]
    common_years = [year for year in range(1752, 10000) if year not in range(1752, 10000, 4)]
    # Seed the random number generator
    random.seed()
    # Choose 10 year numbers at random
    test_cases = random.choices(common_years, k=4) + random.choices(four_years, k=3) + random.choices(one_hundred_years, k=2) + random.choices(four_hundred_years, k=1)

    # Testing the student program against standard solution
    print("Start testing...\n")
    # Record the number of passed test cases, initialized to 0
    passed_cases = 0
    for year in test_cases:
        print("Testing the year of {}...\n".format(year), end='')
        # Run the student program and collect the output
        try:
            student_output = subprocess.run(os.path.dirname(os.path.abspath(__file__)) + "/../HW1/Debug/HW1 " + str(year), stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True, shell=True, universal_newlines=True).stdout.split("CALENDAR\n", 1)
        except subprocess.CalledProcessError as err:
            print(err.stderr, end='')
            return
        # Run the standard solution and collect the output
        try:
            standard_output = subprocess.run(os.path.dirname(os.path.abspath(__file__)) + "/HW1_std_soln", input=str(year), stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True, shell=True, universal_newlines=True).stdout
        except subprocess.CalledProcessError as err:
            print(err.stderr, end='')
            return
        # Verify that 'CALENDAR' is included in the student output
        if len(student_output) < 2:
            print("CALENDAR not found")
            return
        # Compare the student output with the standard output character by character
        production_output = student_output[1]
        if production_output == standard_output:
            print("PASSED\n")
            passed_cases += 1
        else:
        	# Print the difference if the outputs are different
            print("FAILED")
            diff = difflib.ndiff(production_output.splitlines(keepends=True), standard_output.splitlines(keepends=True))
            print(''.join(diff))

    # Print the results of the autograder
    print("You passed {}/10 test case(s).".format(passed_cases))


# Call the main program automatically
if __name__ == "__main__":
    main()
